import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.furlough_report_line import FurloughReportLine
from ..models.item import Item
from ..models.pay_periods import PayPeriods
from ..models.report import Report
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="FurloughReport")


@attr.s(auto_attribs=True)
class FurloughReport:
    """
    Attributes:
        max_claim_per_employee (Union[Unset, float]):
        percentage_of_ni_and_pension_to_claim (Union[Unset, float]):
        govt_contrib_rate (Union[Unset, float]):
        company_name (Union[Unset, None, str]):
        employer_reference (Union[Unset, None, str]):
        company_crn (Union[Unset, None, str]):
        ct_utr (Union[Unset, None, str]):
        sa_utr (Union[Unset, None, str]):
        claim_period_start_date (Union[Unset, datetime.date]):
        claim_period_end_date (Union[Unset, datetime.date]):
        number_of_employees_being_furloughed (Union[Unset, int]):
        total_claim_amount (Union[Unset, float]):
        total_gross_pay (Union[Unset, float]):
        amount_claimed_for_gross_pay_to_employees_on_furlough_for_the_period (Union[Unset, float]):
        amount_claimed_for_employer_ni_cs_contributions_for_furloughed_employees (Union[Unset, float]):
        amount_claimed_for_employers_auto_enrolment_pension_costs_for_furloughed_employees (Union[Unset, float]):
        lines (Union[Unset, None, List[FurloughReportLine]]):
        bank_account_number (Union[Unset, None, str]):
        bank_sort_code (Union[Unset, None, str]):
        bank_account_holders_first_name (Union[Unset, None, str]):
        bank_account_holders_last_name (Union[Unset, None, str]):
        bank_account_holders_address (Union[Unset, None, str]):
        building_society_roll_number (Union[Unset, None, str]):
        company_address (Union[Unset, None, str]):
        contact_name (Union[Unset, None, str]):
        contact_number (Union[Unset, None, str]):
        employer (Union[Unset, Item]):
        pay_period (Union[Unset, PayPeriods]):
        ordinal (Union[Unset, int]):
        period (Union[Unset, int]):
        period_to (Union[Unset, int]):
        start_period_name (Union[Unset, None, str]):
        end_period_name (Union[Unset, None, str]):
        start_date (Union[Unset, datetime.date]):
        end_date (Union[Unset, datetime.date]):
        report (Union[Unset, Report]):
        tax_year (Union[Unset, TaxYear]):
    """

    max_claim_per_employee: Union[Unset, float] = UNSET
    percentage_of_ni_and_pension_to_claim: Union[Unset, float] = UNSET
    govt_contrib_rate: Union[Unset, float] = UNSET
    company_name: Union[Unset, None, str] = UNSET
    employer_reference: Union[Unset, None, str] = UNSET
    company_crn: Union[Unset, None, str] = UNSET
    ct_utr: Union[Unset, None, str] = UNSET
    sa_utr: Union[Unset, None, str] = UNSET
    claim_period_start_date: Union[Unset, datetime.date] = UNSET
    claim_period_end_date: Union[Unset, datetime.date] = UNSET
    number_of_employees_being_furloughed: Union[Unset, int] = UNSET
    total_claim_amount: Union[Unset, float] = UNSET
    total_gross_pay: Union[Unset, float] = UNSET
    amount_claimed_for_gross_pay_to_employees_on_furlough_for_the_period: Union[
        Unset, float
    ] = UNSET
    amount_claimed_for_employer_ni_cs_contributions_for_furloughed_employees: Union[
        Unset, float
    ] = UNSET
    amount_claimed_for_employers_auto_enrolment_pension_costs_for_furloughed_employees: Union[
        Unset, float
    ] = UNSET
    lines: Union[Unset, None, List[FurloughReportLine]] = UNSET
    bank_account_number: Union[Unset, None, str] = UNSET
    bank_sort_code: Union[Unset, None, str] = UNSET
    bank_account_holders_first_name: Union[Unset, None, str] = UNSET
    bank_account_holders_last_name: Union[Unset, None, str] = UNSET
    bank_account_holders_address: Union[Unset, None, str] = UNSET
    building_society_roll_number: Union[Unset, None, str] = UNSET
    company_address: Union[Unset, None, str] = UNSET
    contact_name: Union[Unset, None, str] = UNSET
    contact_number: Union[Unset, None, str] = UNSET
    employer: Union[Unset, Item] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    ordinal: Union[Unset, int] = UNSET
    period: Union[Unset, int] = UNSET
    period_to: Union[Unset, int] = UNSET
    start_period_name: Union[Unset, None, str] = UNSET
    end_period_name: Union[Unset, None, str] = UNSET
    start_date: Union[Unset, datetime.date] = UNSET
    end_date: Union[Unset, datetime.date] = UNSET
    report: Union[Unset, Report] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        max_claim_per_employee = self.max_claim_per_employee
        percentage_of_ni_and_pension_to_claim = (
            self.percentage_of_ni_and_pension_to_claim
        )
        govt_contrib_rate = self.govt_contrib_rate
        company_name = self.company_name
        employer_reference = self.employer_reference
        company_crn = self.company_crn
        ct_utr = self.ct_utr
        sa_utr = self.sa_utr
        claim_period_start_date: Union[Unset, str] = UNSET
        if not isinstance(self.claim_period_start_date, Unset):
            claim_period_start_date = self.claim_period_start_date.isoformat()

        claim_period_end_date: Union[Unset, str] = UNSET
        if not isinstance(self.claim_period_end_date, Unset):
            claim_period_end_date = self.claim_period_end_date.isoformat()

        number_of_employees_being_furloughed = self.number_of_employees_being_furloughed
        total_claim_amount = self.total_claim_amount
        total_gross_pay = self.total_gross_pay
        amount_claimed_for_gross_pay_to_employees_on_furlough_for_the_period = (
            self.amount_claimed_for_gross_pay_to_employees_on_furlough_for_the_period
        )
        amount_claimed_for_employer_ni_cs_contributions_for_furloughed_employees = (
            self.amount_claimed_for_employer_ni_cs_contributions_for_furloughed_employees
        )
        amount_claimed_for_employers_auto_enrolment_pension_costs_for_furloughed_employees = (
            self.amount_claimed_for_employers_auto_enrolment_pension_costs_for_furloughed_employees
        )
        lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.lines, Unset):
            if self.lines is None:
                lines = None
            else:
                lines = []
                for lines_item_data in self.lines:
                    lines_item = lines_item_data.to_dict()

                    lines.append(lines_item)

        bank_account_number = self.bank_account_number
        bank_sort_code = self.bank_sort_code
        bank_account_holders_first_name = self.bank_account_holders_first_name
        bank_account_holders_last_name = self.bank_account_holders_last_name
        bank_account_holders_address = self.bank_account_holders_address
        building_society_roll_number = self.building_society_roll_number
        company_address = self.company_address
        contact_name = self.contact_name
        contact_number = self.contact_number
        employer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer, Unset):
            employer = self.employer.to_dict()

        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        ordinal = self.ordinal
        period = self.period
        period_to = self.period_to
        start_period_name = self.start_period_name
        end_period_name = self.end_period_name
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        report: Union[Unset, str] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.value

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if max_claim_per_employee is not UNSET:
            field_dict["maxClaimPerEmployee"] = max_claim_per_employee
        if percentage_of_ni_and_pension_to_claim is not UNSET:
            field_dict[
                "percentageOfNIAndPensionToClaim"
            ] = percentage_of_ni_and_pension_to_claim
        if govt_contrib_rate is not UNSET:
            field_dict["govtContribRate"] = govt_contrib_rate
        if company_name is not UNSET:
            field_dict["companyName"] = company_name
        if employer_reference is not UNSET:
            field_dict["employerReference"] = employer_reference
        if company_crn is not UNSET:
            field_dict["companyCrn"] = company_crn
        if ct_utr is not UNSET:
            field_dict["ctUtr"] = ct_utr
        if sa_utr is not UNSET:
            field_dict["saUtr"] = sa_utr
        if claim_period_start_date is not UNSET:
            field_dict["claimPeriodStartDate"] = claim_period_start_date
        if claim_period_end_date is not UNSET:
            field_dict["claimPeriodEndDate"] = claim_period_end_date
        if number_of_employees_being_furloughed is not UNSET:
            field_dict[
                "numberOfEmployeesBeingFurloughed"
            ] = number_of_employees_being_furloughed
        if total_claim_amount is not UNSET:
            field_dict["totalClaimAmount"] = total_claim_amount
        if total_gross_pay is not UNSET:
            field_dict["totalGrossPay"] = total_gross_pay
        if (
            amount_claimed_for_gross_pay_to_employees_on_furlough_for_the_period
            is not UNSET
        ):
            field_dict[
                "amountClaimedForGrossPayToEmployeesOnFurloughForThePeriod"
            ] = amount_claimed_for_gross_pay_to_employees_on_furlough_for_the_period
        if (
            amount_claimed_for_employer_ni_cs_contributions_for_furloughed_employees
            is not UNSET
        ):
            field_dict[
                "amountClaimedForEmployerNICsContributionsForFurloughedEmployees"
            ] = amount_claimed_for_employer_ni_cs_contributions_for_furloughed_employees
        if (
            amount_claimed_for_employers_auto_enrolment_pension_costs_for_furloughed_employees
            is not UNSET
        ):
            field_dict[
                "amountClaimedForEmployersAutoEnrolmentPensionCostsForFurloughedEmployees"
            ] = amount_claimed_for_employers_auto_enrolment_pension_costs_for_furloughed_employees
        if lines is not UNSET:
            field_dict["lines"] = lines
        if bank_account_number is not UNSET:
            field_dict["bankAccountNumber"] = bank_account_number
        if bank_sort_code is not UNSET:
            field_dict["bankSortCode"] = bank_sort_code
        if bank_account_holders_first_name is not UNSET:
            field_dict["bankAccountHoldersFirstName"] = bank_account_holders_first_name
        if bank_account_holders_last_name is not UNSET:
            field_dict["bankAccountHoldersLastName"] = bank_account_holders_last_name
        if bank_account_holders_address is not UNSET:
            field_dict["bankAccountHoldersAddress"] = bank_account_holders_address
        if building_society_roll_number is not UNSET:
            field_dict["buildingSocietyRollNumber"] = building_society_roll_number
        if company_address is not UNSET:
            field_dict["companyAddress"] = company_address
        if contact_name is not UNSET:
            field_dict["contactName"] = contact_name
        if contact_number is not UNSET:
            field_dict["contactNumber"] = contact_number
        if employer is not UNSET:
            field_dict["employer"] = employer
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if period is not UNSET:
            field_dict["period"] = period
        if period_to is not UNSET:
            field_dict["periodTo"] = period_to
        if start_period_name is not UNSET:
            field_dict["startPeriodName"] = start_period_name
        if end_period_name is not UNSET:
            field_dict["endPeriodName"] = end_period_name
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if report is not UNSET:
            field_dict["report"] = report
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        max_claim_per_employee = d.pop("maxClaimPerEmployee", UNSET)

        percentage_of_ni_and_pension_to_claim = d.pop(
            "percentageOfNIAndPensionToClaim", UNSET
        )

        govt_contrib_rate = d.pop("govtContribRate", UNSET)

        company_name = d.pop("companyName", UNSET)

        employer_reference = d.pop("employerReference", UNSET)

        company_crn = d.pop("companyCrn", UNSET)

        ct_utr = d.pop("ctUtr", UNSET)

        sa_utr = d.pop("saUtr", UNSET)

        _claim_period_start_date = d.pop("claimPeriodStartDate", UNSET)
        claim_period_start_date: Union[Unset, datetime.date]
        if isinstance(_claim_period_start_date, Unset):
            claim_period_start_date = UNSET
        else:
            claim_period_start_date = isoparse(_claim_period_start_date).date()

        _claim_period_end_date = d.pop("claimPeriodEndDate", UNSET)
        claim_period_end_date: Union[Unset, datetime.date]
        if isinstance(_claim_period_end_date, Unset):
            claim_period_end_date = UNSET
        else:
            claim_period_end_date = isoparse(_claim_period_end_date).date()

        number_of_employees_being_furloughed = d.pop(
            "numberOfEmployeesBeingFurloughed", UNSET
        )

        total_claim_amount = d.pop("totalClaimAmount", UNSET)

        total_gross_pay = d.pop("totalGrossPay", UNSET)

        amount_claimed_for_gross_pay_to_employees_on_furlough_for_the_period = d.pop(
            "amountClaimedForGrossPayToEmployeesOnFurloughForThePeriod", UNSET
        )

        amount_claimed_for_employer_ni_cs_contributions_for_furloughed_employees = (
            d.pop(
                "amountClaimedForEmployerNICsContributionsForFurloughedEmployees", UNSET
            )
        )

        amount_claimed_for_employers_auto_enrolment_pension_costs_for_furloughed_employees = d.pop(
            "amountClaimedForEmployersAutoEnrolmentPensionCostsForFurloughedEmployees",
            UNSET,
        )

        lines = []
        _lines = d.pop("lines", UNSET)
        for lines_item_data in _lines or []:
            lines_item = FurloughReportLine.from_dict(lines_item_data)

            lines.append(lines_item)

        bank_account_number = d.pop("bankAccountNumber", UNSET)

        bank_sort_code = d.pop("bankSortCode", UNSET)

        bank_account_holders_first_name = d.pop("bankAccountHoldersFirstName", UNSET)

        bank_account_holders_last_name = d.pop("bankAccountHoldersLastName", UNSET)

        bank_account_holders_address = d.pop("bankAccountHoldersAddress", UNSET)

        building_society_roll_number = d.pop("buildingSocietyRollNumber", UNSET)

        company_address = d.pop("companyAddress", UNSET)

        contact_name = d.pop("contactName", UNSET)

        contact_number = d.pop("contactNumber", UNSET)

        _employer = d.pop("employer", UNSET)
        employer: Union[Unset, Item]
        if isinstance(_employer, Unset):
            employer = UNSET
        else:
            employer = Item.from_dict(_employer)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        ordinal = d.pop("ordinal", UNSET)

        period = d.pop("period", UNSET)

        period_to = d.pop("periodTo", UNSET)

        start_period_name = d.pop("startPeriodName", UNSET)

        end_period_name = d.pop("endPeriodName", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.date]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.date]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date).date()

        _report = d.pop("report", UNSET)
        report: Union[Unset, Report]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = Report(_report)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        furlough_report = cls(
            max_claim_per_employee=max_claim_per_employee,
            percentage_of_ni_and_pension_to_claim=percentage_of_ni_and_pension_to_claim,
            govt_contrib_rate=govt_contrib_rate,
            company_name=company_name,
            employer_reference=employer_reference,
            company_crn=company_crn,
            ct_utr=ct_utr,
            sa_utr=sa_utr,
            claim_period_start_date=claim_period_start_date,
            claim_period_end_date=claim_period_end_date,
            number_of_employees_being_furloughed=number_of_employees_being_furloughed,
            total_claim_amount=total_claim_amount,
            total_gross_pay=total_gross_pay,
            amount_claimed_for_gross_pay_to_employees_on_furlough_for_the_period=amount_claimed_for_gross_pay_to_employees_on_furlough_for_the_period,
            amount_claimed_for_employer_ni_cs_contributions_for_furloughed_employees=amount_claimed_for_employer_ni_cs_contributions_for_furloughed_employees,
            amount_claimed_for_employers_auto_enrolment_pension_costs_for_furloughed_employees=amount_claimed_for_employers_auto_enrolment_pension_costs_for_furloughed_employees,
            lines=lines,
            bank_account_number=bank_account_number,
            bank_sort_code=bank_sort_code,
            bank_account_holders_first_name=bank_account_holders_first_name,
            bank_account_holders_last_name=bank_account_holders_last_name,
            bank_account_holders_address=bank_account_holders_address,
            building_society_roll_number=building_society_roll_number,
            company_address=company_address,
            contact_name=contact_name,
            contact_number=contact_number,
            employer=employer,
            pay_period=pay_period,
            ordinal=ordinal,
            period=period,
            period_to=period_to,
            start_period_name=start_period_name,
            end_period_name=end_period_name,
            start_date=start_date,
            end_date=end_date,
            report=report,
            tax_year=tax_year,
        )

        return furlough_report



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployeeName")


@attr.s(auto_attribs=True)
class PapdisEmployeeName:
    """
    Attributes:
        title (Union[Unset, None, str]): [readonly]
        forename1 (Union[Unset, None, str]): [readonly]
        forename2 (Union[Unset, None, str]): [readonly]
        surname (Union[Unset, None, str]): [readonly]
    """

    title: Union[Unset, None, str] = UNSET
    forename1: Union[Unset, None, str] = UNSET
    forename2: Union[Unset, None, str] = UNSET
    surname: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        forename1 = self.forename1
        forename2 = self.forename2
        surname = self.surname

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if title is not UNSET:
            field_dict["title"] = title
        if forename1 is not UNSET:
            field_dict["forename1"] = forename1
        if forename2 is not UNSET:
            field_dict["forename2"] = forename2
        if surname is not UNSET:
            field_dict["surname"] = surname

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title", UNSET)

        forename1 = d.pop("forename1", UNSET)

        forename2 = d.pop("forename2", UNSET)

        surname = d.pop("surname", UNSET)

        papdis_employee_name = cls(
            title=title,
            forename1=forename1,
            forename2=forename2,
            surname=surname,
        )

        return papdis_employee_name



from enum import Enum


class HoursNormallyWorked(str, Enum):
    LESSTHAN16 = "LessThan16"
    MORETHAN16 = "MoreThan16"
    MORETHAN24 = "MoreThan24"
    MORETHAN30 = "MoreThan30"
    NOTREGULAR = "NotRegular"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.pay_basis import PayBasis
from ..models.pay_line import PayLine
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayOptionsImport")


@attr.s(auto_attribs=True)
class PayOptionsImport:
    """This object is used to import payment information for a payrun entry

    Attributes:
        employer_identifier (Union[Unset, None, str]): Optional. But if one entry has it then all must.
            Allows you to import to multiple employers by specifying the Employers AlternativeIdentifier
        payroll_code (Union[Unset, None, str]): The payroll code of the employee to update
        pay_amount (Union[Unset, float]): The amount the Employee is regularly paid each period
        basis (Union[Unset, PayBasis]):
        pay_code (Union[Unset, None, str]): If you want to override the PayCode used for the Basic Pay then set the code
            here, otherwise leave this blank and the default will be used.
        pay_amount_multiplier (Union[Unset, float]): This property is irrelevant if the basis is Monthly.
            But if the basis is Daily or Hourly then this property sets how many days/hours the employee should be paid for
            in the period.
        note (Union[Unset, None, str]): Any note that you'd like to appear on the payslip
        tags (Union[Unset, None, List[str]]):
        lines (Union[Unset, None, List[PayLine]]):
    """

    employer_identifier: Union[Unset, None, str] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    pay_amount: Union[Unset, float] = UNSET
    basis: Union[Unset, PayBasis] = UNSET
    pay_code: Union[Unset, None, str] = UNSET
    pay_amount_multiplier: Union[Unset, float] = UNSET
    note: Union[Unset, None, str] = UNSET
    tags: Union[Unset, None, List[str]] = UNSET
    lines: Union[Unset, None, List[PayLine]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employer_identifier = self.employer_identifier
        payroll_code = self.payroll_code
        pay_amount = self.pay_amount
        basis: Union[Unset, str] = UNSET
        if not isinstance(self.basis, Unset):
            basis = self.basis.value

        pay_code = self.pay_code
        pay_amount_multiplier = self.pay_amount_multiplier
        note = self.note
        tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            if self.tags is None:
                tags = None
            else:
                tags = self.tags

        lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.lines, Unset):
            if self.lines is None:
                lines = None
            else:
                lines = []
                for lines_item_data in self.lines:
                    lines_item = lines_item_data.to_dict()

                    lines.append(lines_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employer_identifier is not UNSET:
            field_dict["employerIdentifier"] = employer_identifier
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if pay_amount is not UNSET:
            field_dict["payAmount"] = pay_amount
        if basis is not UNSET:
            field_dict["basis"] = basis
        if pay_code is not UNSET:
            field_dict["payCode"] = pay_code
        if pay_amount_multiplier is not UNSET:
            field_dict["payAmountMultiplier"] = pay_amount_multiplier
        if note is not UNSET:
            field_dict["note"] = note
        if tags is not UNSET:
            field_dict["tags"] = tags
        if lines is not UNSET:
            field_dict["lines"] = lines

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employer_identifier = d.pop("employerIdentifier", UNSET)

        payroll_code = d.pop("payrollCode", UNSET)

        pay_amount = d.pop("payAmount", UNSET)

        _basis = d.pop("basis", UNSET)
        basis: Union[Unset, PayBasis]
        if isinstance(_basis, Unset):
            basis = UNSET
        else:
            basis = PayBasis(_basis)

        pay_code = d.pop("payCode", UNSET)

        pay_amount_multiplier = d.pop("payAmountMultiplier", UNSET)

        note = d.pop("note", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        lines = []
        _lines = d.pop("lines", UNSET)
        for lines_item_data in _lines or []:
            lines_item = PayLine.from_dict(lines_item_data)

            lines.append(lines_item)

        pay_options_import = cls(
            employer_identifier=employer_identifier,
            payroll_code=payroll_code,
            pay_amount=pay_amount,
            basis=basis,
            pay_code=pay_code,
            pay_amount_multiplier=pay_amount_multiplier,
            note=note,
            tags=tags,
            lines=lines,
        )

        return pay_options_import



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.background_task_status import BackgroundTaskStatus
from ..models.pay_periods import PayPeriods
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="PensionContributionsSubmission")


@attr.s(auto_attribs=True)
class PensionContributionsSubmission:
    """This model is used to track submission of Pension Contributions to an external data provider.

    Attributes:
        employer_id (Union[Unset, int]):
        pay_run_id (Union[Unset, int]):
        pay_period (Union[Unset, PayPeriods]):
        ordinal (Union[Unset, int]):
        period (Union[Unset, int]):
        tax_year (Union[Unset, TaxYear]):
        scheme_id (Union[Unset, int]):
        status (Union[Unset, BackgroundTaskStatus]):
        external_id (Union[Unset, None, str]):
        status_message (Union[Unset, None, str]):
        submission_data (Union[Unset, Any]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    employer_id: Union[Unset, int] = UNSET
    pay_run_id: Union[Unset, int] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    ordinal: Union[Unset, int] = UNSET
    period: Union[Unset, int] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    scheme_id: Union[Unset, int] = UNSET
    status: Union[Unset, BackgroundTaskStatus] = UNSET
    external_id: Union[Unset, None, str] = UNSET
    status_message: Union[Unset, None, str] = UNSET
    submission_data: Union[Unset, Any] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employer_id = self.employer_id
        pay_run_id = self.pay_run_id
        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        ordinal = self.ordinal
        period = self.period
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        scheme_id = self.scheme_id
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        external_id = self.external_id
        status_message = self.status_message
        submission_data = self.submission_data
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employer_id is not UNSET:
            field_dict["employerId"] = employer_id
        if pay_run_id is not UNSET:
            field_dict["payRunId"] = pay_run_id
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if period is not UNSET:
            field_dict["period"] = period
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if scheme_id is not UNSET:
            field_dict["schemeId"] = scheme_id
        if status is not UNSET:
            field_dict["status"] = status
        if external_id is not UNSET:
            field_dict["externalId"] = external_id
        if status_message is not UNSET:
            field_dict["statusMessage"] = status_message
        if submission_data is not UNSET:
            field_dict["submissionData"] = submission_data
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employer_id = d.pop("employerId", UNSET)

        pay_run_id = d.pop("payRunId", UNSET)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        ordinal = d.pop("ordinal", UNSET)

        period = d.pop("period", UNSET)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        scheme_id = d.pop("schemeId", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, BackgroundTaskStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = BackgroundTaskStatus(_status)

        external_id = d.pop("externalId", UNSET)

        status_message = d.pop("statusMessage", UNSET)

        submission_data = d.pop("submissionData", UNSET)

        id = d.pop("id", UNSET)

        pension_contributions_submission = cls(
            employer_id=employer_id,
            pay_run_id=pay_run_id,
            pay_period=pay_period,
            ordinal=ordinal,
            period=period,
            tax_year=tax_year,
            scheme_id=scheme_id,
            status=status,
            external_id=external_id,
            status_message=status_message,
            submission_data=submission_data,
            id=id,
        )

        return pension_contributions_submission



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.employer_item import EmployerItem
from ..models.tenant_item import TenantItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserAuthorization")


@attr.s(auto_attribs=True)
class UserAuthorization:
    """This model provides information about what the User is able to access.
    This would usually just be a list of Employers. But if the user is an administrator for a White Label instance then
    this will be shown in the list of Tenants.

        Attributes:
            employers (Union[Unset, None, List[EmployerItem]]): [readonly] A list of any Employers that the user can access
            tenants (Union[Unset, None, List[TenantItem]]): [readonly] A list of any Tenants that the user can administrate
    """

    employers: Union[Unset, None, List[EmployerItem]] = UNSET
    tenants: Union[Unset, None, List[TenantItem]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employers: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.employers, Unset):
            if self.employers is None:
                employers = None
            else:
                employers = []
                for employers_item_data in self.employers:
                    employers_item = employers_item_data.to_dict()

                    employers.append(employers_item)

        tenants: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tenants, Unset):
            if self.tenants is None:
                tenants = None
            else:
                tenants = []
                for tenants_item_data in self.tenants:
                    tenants_item = tenants_item_data.to_dict()

                    tenants.append(tenants_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employers is not UNSET:
            field_dict["employers"] = employers
        if tenants is not UNSET:
            field_dict["tenants"] = tenants

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employers = []
        _employers = d.pop("employers", UNSET)
        for employers_item_data in _employers or []:
            employers_item = EmployerItem.from_dict(employers_item_data)

            employers.append(employers_item)

        tenants = []
        _tenants = d.pop("tenants", UNSET)
        for tenants_item_data in _tenants or []:
            tenants_item = TenantItem.from_dict(tenants_item_data)

            tenants.append(tenants_item)

        user_authorization = cls(
            employers=employers,
            tenants=tenants,
        )

        return user_authorization



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.papdis_employee import PapdisEmployee
from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisPayrollPeriod")


@attr.s(auto_attribs=True)
class PapdisPayrollPeriod:
    """
    Attributes:
        pay_period_start_date (Union[Unset, datetime.date]): [readonly]
        pay_period_end_date (Union[Unset, datetime.date]): [readonly]
        contribution_deduction_date (Union[Unset, datetime.date]): [readonly]
        frequency_code (Union[Unset, None, str]): [readonly]
        tax_period (Union[Unset, int]): [readonly]
        pay_reference_start_date (Union[Unset, datetime.date]): [readonly]
        pay_reference_end_date (Union[Unset, datetime.date]): [readonly]
        employees (Union[Unset, None, List[PapdisEmployee]]): [readonly] Employees and contributions made in this period
    """

    pay_period_start_date: Union[Unset, datetime.date] = UNSET
    pay_period_end_date: Union[Unset, datetime.date] = UNSET
    contribution_deduction_date: Union[Unset, datetime.date] = UNSET
    frequency_code: Union[Unset, None, str] = UNSET
    tax_period: Union[Unset, int] = UNSET
    pay_reference_start_date: Union[Unset, datetime.date] = UNSET
    pay_reference_end_date: Union[Unset, datetime.date] = UNSET
    employees: Union[Unset, None, List[PapdisEmployee]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pay_period_start_date: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period_start_date, Unset):
            pay_period_start_date = self.pay_period_start_date.isoformat()

        pay_period_end_date: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period_end_date, Unset):
            pay_period_end_date = self.pay_period_end_date.isoformat()

        contribution_deduction_date: Union[Unset, str] = UNSET
        if not isinstance(self.contribution_deduction_date, Unset):
            contribution_deduction_date = self.contribution_deduction_date.isoformat()

        frequency_code = self.frequency_code
        tax_period = self.tax_period
        pay_reference_start_date: Union[Unset, str] = UNSET
        if not isinstance(self.pay_reference_start_date, Unset):
            pay_reference_start_date = self.pay_reference_start_date.isoformat()

        pay_reference_end_date: Union[Unset, str] = UNSET
        if not isinstance(self.pay_reference_end_date, Unset):
            pay_reference_end_date = self.pay_reference_end_date.isoformat()

        employees: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.employees, Unset):
            if self.employees is None:
                employees = None
            else:
                employees = []
                for employees_item_data in self.employees:
                    employees_item = employees_item_data.to_dict()

                    employees.append(employees_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pay_period_start_date is not UNSET:
            field_dict["payPeriodStartDate"] = pay_period_start_date
        if pay_period_end_date is not UNSET:
            field_dict["payPeriodEndDate"] = pay_period_end_date
        if contribution_deduction_date is not UNSET:
            field_dict["contributionDeductionDate"] = contribution_deduction_date
        if frequency_code is not UNSET:
            field_dict["frequencyCode"] = frequency_code
        if tax_period is not UNSET:
            field_dict["taxPeriod"] = tax_period
        if pay_reference_start_date is not UNSET:
            field_dict["payReferenceStartDate"] = pay_reference_start_date
        if pay_reference_end_date is not UNSET:
            field_dict["payReferenceEndDate"] = pay_reference_end_date
        if employees is not UNSET:
            field_dict["employees"] = employees

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _pay_period_start_date = d.pop("payPeriodStartDate", UNSET)
        pay_period_start_date: Union[Unset, datetime.date]
        if isinstance(_pay_period_start_date, Unset):
            pay_period_start_date = UNSET
        else:
            pay_period_start_date = isoparse(_pay_period_start_date).date()

        _pay_period_end_date = d.pop("payPeriodEndDate", UNSET)
        pay_period_end_date: Union[Unset, datetime.date]
        if isinstance(_pay_period_end_date, Unset):
            pay_period_end_date = UNSET
        else:
            pay_period_end_date = isoparse(_pay_period_end_date).date()

        _contribution_deduction_date = d.pop("contributionDeductionDate", UNSET)
        contribution_deduction_date: Union[Unset, datetime.date]
        if isinstance(_contribution_deduction_date, Unset):
            contribution_deduction_date = UNSET
        else:
            contribution_deduction_date = isoparse(_contribution_deduction_date).date()

        frequency_code = d.pop("frequencyCode", UNSET)

        tax_period = d.pop("taxPeriod", UNSET)

        _pay_reference_start_date = d.pop("payReferenceStartDate", UNSET)
        pay_reference_start_date: Union[Unset, datetime.date]
        if isinstance(_pay_reference_start_date, Unset):
            pay_reference_start_date = UNSET
        else:
            pay_reference_start_date = isoparse(_pay_reference_start_date).date()

        _pay_reference_end_date = d.pop("payReferenceEndDate", UNSET)
        pay_reference_end_date: Union[Unset, datetime.date]
        if isinstance(_pay_reference_end_date, Unset):
            pay_reference_end_date = UNSET
        else:
            pay_reference_end_date = isoparse(_pay_reference_end_date).date()

        employees = []
        _employees = d.pop("employees", UNSET)
        for employees_item_data in _employees or []:
            employees_item = PapdisEmployee.from_dict(employees_item_data)

            employees.append(employees_item)

        papdis_payroll_period = cls(
            pay_period_start_date=pay_period_start_date,
            pay_period_end_date=pay_period_end_date,
            contribution_deduction_date=contribution_deduction_date,
            frequency_code=frequency_code,
            tax_period=tax_period,
            pay_reference_start_date=pay_reference_start_date,
            pay_reference_end_date=pay_reference_end_date,
            employees=employees,
        )

        return papdis_payroll_period



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.cost_breakdown_breakdown import CostBreakdownBreakdown
from ..types import UNSET, Unset

T = TypeVar("T", bound="CostBreakdown")


@attr.s(auto_attribs=True)
class CostBreakdown:
    """Represents the breakdown of a PayRun cost.
    The Value if calculated using the breakdown of the cost.

        Attributes:
            value (Union[Unset, float]): The value of cost
            breakdown (Union[Unset, None, CostBreakdownBreakdown]): Breakdown of the cost
    """

    value: Union[Unset, float] = UNSET
    breakdown: Union[Unset, None, CostBreakdownBreakdown] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        value = self.value
        breakdown: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.breakdown, Unset):
            breakdown = self.breakdown.to_dict() if self.breakdown else None

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if value is not UNSET:
            field_dict["value"] = value
        if breakdown is not UNSET:
            field_dict["breakdown"] = breakdown

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        value = d.pop("value", UNSET)

        _breakdown = d.pop("breakdown", UNSET)
        breakdown: Union[Unset, None, CostBreakdownBreakdown]
        if _breakdown is None:
            breakdown = None
        elif isinstance(_breakdown, Unset):
            breakdown = UNSET
        else:
            breakdown = CostBreakdownBreakdown.from_dict(_breakdown)

        cost_breakdown = cls(
            value=value,
            breakdown=breakdown,
        )

        return cost_breakdown



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.mail_settings import MailSettings
from ..models.tenant_billing_settings import TenantBillingSettings
from ..models.tenant_html_insertion import TenantHtmlInsertion
from ..types import UNSET, Unset

T = TypeVar("T", bound="Tenant")


@attr.s(auto_attribs=True)
class Tenant:
    """The Tenant model represents the brand that provides the account.
    This is used by our White Label partners to manage and brand their user accounts.
    Unless you are an admin for a White Label account you'll have no interest in this model.

        Attributes:
            brand_code (Union[Unset, None, str]): [readonly]
            app_name (Union[Unset, None, str]):
            home_url (Union[Unset, None, str]):
            head_content (Union[Unset, None, str]):
            log_out_url (Union[Unset, None, str]):
            login_img_url (Union[Unset, None, str]): [readonly]
            home_img_url (Union[Unset, None, str]): [readonly]
            fav_icon (Union[Unset, None, str]): [readonly]
            css_colors_file (Union[Unset, None, str]): [readonly]
            css_file (Union[Unset, None, str]): [readonly]
            mailing_list (Union[Unset, bool]):
            html_insertions (Union[Unset, None, List[TenantHtmlInsertion]]):
            mail_settings (Union[Unset, MailSettings]): Determines the settings used when the Employer sends emails.
                If CustomiseSmtpSettings is false then SmtpSettings will be null and our default internal settings will be used;
            signup_url (Union[Unset, None, str]):
            terms_url (Union[Unset, None, str]):
            help_url (Union[Unset, None, str]):
            support_email (Union[Unset, None, str]):
            new_user_signup_email (Union[Unset, None, str]):
            approve_new_users (Union[Unset, bool]):
            enable_bureau_features (Union[Unset, bool]):
            require_dd_mandate_before_allowing_billable_activity (Union[Unset, bool]):
            enable_omnipresent_users (Union[Unset, bool]):
            tenant_owns_billing (Union[Unset, bool]): [readonly]
            billing_settings (Union[Unset, TenantBillingSettings]):
            id (Union[Unset, str]): [readonly] The unique id of the object
    """

    brand_code: Union[Unset, None, str] = UNSET
    app_name: Union[Unset, None, str] = UNSET
    home_url: Union[Unset, None, str] = UNSET
    head_content: Union[Unset, None, str] = UNSET
    log_out_url: Union[Unset, None, str] = UNSET
    login_img_url: Union[Unset, None, str] = UNSET
    home_img_url: Union[Unset, None, str] = UNSET
    fav_icon: Union[Unset, None, str] = UNSET
    css_colors_file: Union[Unset, None, str] = UNSET
    css_file: Union[Unset, None, str] = UNSET
    mailing_list: Union[Unset, bool] = UNSET
    html_insertions: Union[Unset, None, List[TenantHtmlInsertion]] = UNSET
    mail_settings: Union[Unset, MailSettings] = UNSET
    signup_url: Union[Unset, None, str] = UNSET
    terms_url: Union[Unset, None, str] = UNSET
    help_url: Union[Unset, None, str] = UNSET
    support_email: Union[Unset, None, str] = UNSET
    new_user_signup_email: Union[Unset, None, str] = UNSET
    approve_new_users: Union[Unset, bool] = UNSET
    enable_bureau_features: Union[Unset, bool] = UNSET
    require_dd_mandate_before_allowing_billable_activity: Union[Unset, bool] = UNSET
    enable_omnipresent_users: Union[Unset, bool] = UNSET
    tenant_owns_billing: Union[Unset, bool] = UNSET
    billing_settings: Union[Unset, TenantBillingSettings] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        brand_code = self.brand_code
        app_name = self.app_name
        home_url = self.home_url
        head_content = self.head_content
        log_out_url = self.log_out_url
        login_img_url = self.login_img_url
        home_img_url = self.home_img_url
        fav_icon = self.fav_icon
        css_colors_file = self.css_colors_file
        css_file = self.css_file
        mailing_list = self.mailing_list
        html_insertions: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.html_insertions, Unset):
            if self.html_insertions is None:
                html_insertions = None
            else:
                html_insertions = []
                for html_insertions_item_data in self.html_insertions:
                    html_insertions_item = html_insertions_item_data.to_dict()

                    html_insertions.append(html_insertions_item)

        mail_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mail_settings, Unset):
            mail_settings = self.mail_settings.to_dict()

        signup_url = self.signup_url
        terms_url = self.terms_url
        help_url = self.help_url
        support_email = self.support_email
        new_user_signup_email = self.new_user_signup_email
        approve_new_users = self.approve_new_users
        enable_bureau_features = self.enable_bureau_features
        require_dd_mandate_before_allowing_billable_activity = (
            self.require_dd_mandate_before_allowing_billable_activity
        )
        enable_omnipresent_users = self.enable_omnipresent_users
        tenant_owns_billing = self.tenant_owns_billing
        billing_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.billing_settings, Unset):
            billing_settings = self.billing_settings.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if brand_code is not UNSET:
            field_dict["brandCode"] = brand_code
        if app_name is not UNSET:
            field_dict["appName"] = app_name
        if home_url is not UNSET:
            field_dict["homeUrl"] = home_url
        if head_content is not UNSET:
            field_dict["headContent"] = head_content
        if log_out_url is not UNSET:
            field_dict["logOutUrl"] = log_out_url
        if login_img_url is not UNSET:
            field_dict["loginImgUrl"] = login_img_url
        if home_img_url is not UNSET:
            field_dict["homeImgUrl"] = home_img_url
        if fav_icon is not UNSET:
            field_dict["favIcon"] = fav_icon
        if css_colors_file is not UNSET:
            field_dict["cssColorsFile"] = css_colors_file
        if css_file is not UNSET:
            field_dict["cssFile"] = css_file
        if mailing_list is not UNSET:
            field_dict["mailingList"] = mailing_list
        if html_insertions is not UNSET:
            field_dict["htmlInsertions"] = html_insertions
        if mail_settings is not UNSET:
            field_dict["mailSettings"] = mail_settings
        if signup_url is not UNSET:
            field_dict["signupUrl"] = signup_url
        if terms_url is not UNSET:
            field_dict["termsUrl"] = terms_url
        if help_url is not UNSET:
            field_dict["helpUrl"] = help_url
        if support_email is not UNSET:
            field_dict["supportEmail"] = support_email
        if new_user_signup_email is not UNSET:
            field_dict["newUserSignupEmail"] = new_user_signup_email
        if approve_new_users is not UNSET:
            field_dict["approveNewUsers"] = approve_new_users
        if enable_bureau_features is not UNSET:
            field_dict["enableBureauFeatures"] = enable_bureau_features
        if require_dd_mandate_before_allowing_billable_activity is not UNSET:
            field_dict[
                "requireDdMandateBeforeAllowingBillableActivity"
            ] = require_dd_mandate_before_allowing_billable_activity
        if enable_omnipresent_users is not UNSET:
            field_dict["enableOmnipresentUsers"] = enable_omnipresent_users
        if tenant_owns_billing is not UNSET:
            field_dict["tenantOwnsBilling"] = tenant_owns_billing
        if billing_settings is not UNSET:
            field_dict["billingSettings"] = billing_settings
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        brand_code = d.pop("brandCode", UNSET)

        app_name = d.pop("appName", UNSET)

        home_url = d.pop("homeUrl", UNSET)

        head_content = d.pop("headContent", UNSET)

        log_out_url = d.pop("logOutUrl", UNSET)

        login_img_url = d.pop("loginImgUrl", UNSET)

        home_img_url = d.pop("homeImgUrl", UNSET)

        fav_icon = d.pop("favIcon", UNSET)

        css_colors_file = d.pop("cssColorsFile", UNSET)

        css_file = d.pop("cssFile", UNSET)

        mailing_list = d.pop("mailingList", UNSET)

        html_insertions = []
        _html_insertions = d.pop("htmlInsertions", UNSET)
        for html_insertions_item_data in _html_insertions or []:
            html_insertions_item = TenantHtmlInsertion.from_dict(
                html_insertions_item_data
            )

            html_insertions.append(html_insertions_item)

        _mail_settings = d.pop("mailSettings", UNSET)
        mail_settings: Union[Unset, MailSettings]
        if isinstance(_mail_settings, Unset):
            mail_settings = UNSET
        else:
            mail_settings = MailSettings.from_dict(_mail_settings)

        signup_url = d.pop("signupUrl", UNSET)

        terms_url = d.pop("termsUrl", UNSET)

        help_url = d.pop("helpUrl", UNSET)

        support_email = d.pop("supportEmail", UNSET)

        new_user_signup_email = d.pop("newUserSignupEmail", UNSET)

        approve_new_users = d.pop("approveNewUsers", UNSET)

        enable_bureau_features = d.pop("enableBureauFeatures", UNSET)

        require_dd_mandate_before_allowing_billable_activity = d.pop(
            "requireDdMandateBeforeAllowingBillableActivity", UNSET
        )

        enable_omnipresent_users = d.pop("enableOmnipresentUsers", UNSET)

        tenant_owns_billing = d.pop("tenantOwnsBilling", UNSET)

        _billing_settings = d.pop("billingSettings", UNSET)
        billing_settings: Union[Unset, TenantBillingSettings]
        if isinstance(_billing_settings, Unset):
            billing_settings = UNSET
        else:
            billing_settings = TenantBillingSettings.from_dict(_billing_settings)

        id = d.pop("id", UNSET)

        tenant = cls(
            brand_code=brand_code,
            app_name=app_name,
            home_url=home_url,
            head_content=head_content,
            log_out_url=log_out_url,
            login_img_url=login_img_url,
            home_img_url=home_img_url,
            fav_icon=fav_icon,
            css_colors_file=css_colors_file,
            css_file=css_file,
            mailing_list=mailing_list,
            html_insertions=html_insertions,
            mail_settings=mail_settings,
            signup_url=signup_url,
            terms_url=terms_url,
            help_url=help_url,
            support_email=support_email,
            new_user_signup_email=new_user_signup_email,
            approve_new_users=approve_new_users,
            enable_bureau_features=enable_bureau_features,
            require_dd_mandate_before_allowing_billable_activity=require_dd_mandate_before_allowing_billable_activity,
            enable_omnipresent_users=enable_omnipresent_users,
            tenant_owns_billing=tenant_owns_billing,
            billing_settings=billing_settings,
            id=id,
        )

        return tenant



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployeeExit")


@attr.s(auto_attribs=True)
class PapdisEmployeeExit:
    """
    Attributes:
        exit_date (Union[Unset, datetime.date]): [readonly]
        exit_reason_code (Union[Unset, None, int]): [readonly]
        ae_opt_out_date (Union[Unset, None, datetime.date]): [readonly]
        ae_opt_out_reference (Union[Unset, None, str]): [readonly]
    """

    exit_date: Union[Unset, datetime.date] = UNSET
    exit_reason_code: Union[Unset, None, int] = UNSET
    ae_opt_out_date: Union[Unset, None, datetime.date] = UNSET
    ae_opt_out_reference: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        exit_date: Union[Unset, str] = UNSET
        if not isinstance(self.exit_date, Unset):
            exit_date = self.exit_date.isoformat()

        exit_reason_code = self.exit_reason_code
        ae_opt_out_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.ae_opt_out_date, Unset):
            ae_opt_out_date = (
                self.ae_opt_out_date.isoformat() if self.ae_opt_out_date else None
            )

        ae_opt_out_reference = self.ae_opt_out_reference

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if exit_date is not UNSET:
            field_dict["exitDate"] = exit_date
        if exit_reason_code is not UNSET:
            field_dict["exitReasonCode"] = exit_reason_code
        if ae_opt_out_date is not UNSET:
            field_dict["aeOptOutDate"] = ae_opt_out_date
        if ae_opt_out_reference is not UNSET:
            field_dict["aeOptOutReference"] = ae_opt_out_reference

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _exit_date = d.pop("exitDate", UNSET)
        exit_date: Union[Unset, datetime.date]
        if isinstance(_exit_date, Unset):
            exit_date = UNSET
        else:
            exit_date = isoparse(_exit_date).date()

        exit_reason_code = d.pop("exitReasonCode", UNSET)

        _ae_opt_out_date = d.pop("aeOptOutDate", UNSET)
        ae_opt_out_date: Union[Unset, None, datetime.date]
        if _ae_opt_out_date is None:
            ae_opt_out_date = None
        elif isinstance(_ae_opt_out_date, Unset):
            ae_opt_out_date = UNSET
        else:
            ae_opt_out_date = isoparse(_ae_opt_out_date).date()

        ae_opt_out_reference = d.pop("aeOptOutReference", UNSET)

        papdis_employee_exit = cls(
            exit_date=exit_date,
            exit_reason_code=exit_reason_code,
            ae_opt_out_date=ae_opt_out_date,
            ae_opt_out_reference=ae_opt_out_reference,
        )

        return papdis_employee_exit



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="MonthlyMinimum")


@attr.s(auto_attribs=True)
class MonthlyMinimum:
    """
    Attributes:
        enabled (Union[Unset, bool]):
        amount (Union[Unset, float]):
        start_date (Union[Unset, None, datetime.date]):
        end_date (Union[Unset, None, datetime.date]):
        bill_past_end_date (Union[Unset, bool]): If set to true then this user should still be billed even after the End
            Date
    """

    enabled: Union[Unset, bool] = UNSET
    amount: Union[Unset, float] = UNSET
    start_date: Union[Unset, None, datetime.date] = UNSET
    end_date: Union[Unset, None, datetime.date] = UNSET
    bill_past_end_date: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled
        amount = self.amount
        start_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat() if self.start_date else None

        end_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat() if self.end_date else None

        bill_past_end_date = self.bill_past_end_date

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if amount is not UNSET:
            field_dict["amount"] = amount
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if bill_past_end_date is not UNSET:
            field_dict["billPastEndDate"] = bill_past_end_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enabled = d.pop("enabled", UNSET)

        amount = d.pop("amount", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, None, datetime.date]
        if _start_date is None:
            start_date = None
        elif isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, None, datetime.date]
        if _end_date is None:
            end_date = None
        elif isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date).date()

        bill_past_end_date = d.pop("billPastEndDate", UNSET)

        monthly_minimum = cls(
            enabled=enabled,
            amount=amount,
            start_date=start_date,
            end_date=end_date,
            bill_past_end_date=bill_past_end_date,
        )

        return monthly_minimum



from typing import Any, Dict, Optional, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.pay_periods import PayPeriods
from ...models.processing_note import ProcessingNote
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/payrun/{taxYear}/{payPeriod}/{periodNumber}/processingnotes/{id}".format(
        client.base_url,
        employerId=employer_id,
        taxYear=tax_year,
        payPeriod=pay_period,
        periodNumber=period_number,
        id=id,
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["ordinal"] = ordinal

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ProcessingNote]:
    if response.status_code == 200:
        response_200 = ProcessingNote.from_dict(response.json())

        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[ProcessingNote]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[ProcessingNote]:
    """Get a ProcessingNote

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        id (str):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[ProcessingNote]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        id=id,
        client=client,
        ordinal=ordinal,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[ProcessingNote]:
    """Get a ProcessingNote

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        id (str):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[ProcessingNote]
    """

    return sync_detailed(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        id=id,
        client=client,
        ordinal=ordinal,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[ProcessingNote]:
    """Get a ProcessingNote

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        id (str):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[ProcessingNote]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        id=id,
        client=client,
        ordinal=ordinal,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[ProcessingNote]:
    """Get a ProcessingNote

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        id (str):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[ProcessingNote]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            tax_year=tax_year,
            pay_period=pay_period,
            period_number=period_number,
            id=id,
            client=client,
            ordinal=ordinal,
        )
    ).parsed



from enum import Enum


class CISSubContractorType(str, Enum):
    SOLETRADER = "SoleTrader"
    PARTNERSHIP = "Partnership"
    COMPANY = "Company"
    TRUST = "Trust"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EmployerGroup")


@attr.s(auto_attribs=True)
class EmployerGroup:
    """
    Attributes:
        code (str): The unique code for this EmployerGroup
        name (str):
        employer_count (Union[Unset, int]):
        color (Union[Unset, None, str]): A color to used to represent this EmployerGroup, in hex format. ie 'ff0000'
    """

    code: str
    name: str
    employer_count: Union[Unset, int] = UNSET
    color: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        name = self.name
        employer_count = self.employer_count
        color = self.color

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "code": code,
                "name": name,
            }
        )
        if employer_count is not UNSET:
            field_dict["employerCount"] = employer_count
        if color is not UNSET:
            field_dict["color"] = color

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code")

        name = d.pop("name")

        employer_count = d.pop("employerCount", UNSET)

        color = d.pop("color", UNSET)

        employer_group = cls(
            code=code,
            name=name,
            employer_count=employer_count,
            color=color,
        )

        return employer_group



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.employer_item import EmployerItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="EmployerDefaults")


@attr.s(auto_attribs=True)
class EmployerDefaults:
    """When a user creates a new Employer, certain settings can be copied from an existing employer.
    This model determines which employer (if any) settings should be copied from and what should be copied.

        Attributes:
            employer (Union[Unset, EmployerItem]):
            pay_codes (Union[Unset, bool]): If true then PayCodes will be copied from the specified Employer
            csv_mappings (Union[Unset, bool]): If true then CSV Mappings will be copied from the specified Employer
            departments (Union[Unset, bool]): If true then Departments will be copied from the specified Employer
            rti (Union[Unset, bool]): If true then RTI Settings will be copied from the specified Employer
            users (Union[Unset, bool]): If true then Users will be copied from the specified Employer
            hmrc_notice_settings (Union[Unset, bool]): If true then HMRC Notice Settings will be copied from the specified
                Employer
            pay_options (Union[Unset, bool]): If true then Default Pay Options will be copied from the specified Employer.
    """

    employer: Union[Unset, EmployerItem] = UNSET
    pay_codes: Union[Unset, bool] = UNSET
    csv_mappings: Union[Unset, bool] = UNSET
    departments: Union[Unset, bool] = UNSET
    rti: Union[Unset, bool] = UNSET
    users: Union[Unset, bool] = UNSET
    hmrc_notice_settings: Union[Unset, bool] = UNSET
    pay_options: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer, Unset):
            employer = self.employer.to_dict()

        pay_codes = self.pay_codes
        csv_mappings = self.csv_mappings
        departments = self.departments
        rti = self.rti
        users = self.users
        hmrc_notice_settings = self.hmrc_notice_settings
        pay_options = self.pay_options

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employer is not UNSET:
            field_dict["employer"] = employer
        if pay_codes is not UNSET:
            field_dict["payCodes"] = pay_codes
        if csv_mappings is not UNSET:
            field_dict["csvMappings"] = csv_mappings
        if departments is not UNSET:
            field_dict["departments"] = departments
        if rti is not UNSET:
            field_dict["rti"] = rti
        if users is not UNSET:
            field_dict["users"] = users
        if hmrc_notice_settings is not UNSET:
            field_dict["hmrcNoticeSettings"] = hmrc_notice_settings
        if pay_options is not UNSET:
            field_dict["payOptions"] = pay_options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employer = d.pop("employer", UNSET)
        employer: Union[Unset, EmployerItem]
        if isinstance(_employer, Unset):
            employer = UNSET
        else:
            employer = EmployerItem.from_dict(_employer)

        pay_codes = d.pop("payCodes", UNSET)

        csv_mappings = d.pop("csvMappings", UNSET)

        departments = d.pop("departments", UNSET)

        rti = d.pop("rti", UNSET)

        users = d.pop("users", UNSET)

        hmrc_notice_settings = d.pop("hmrcNoticeSettings", UNSET)

        pay_options = d.pop("payOptions", UNSET)

        employer_defaults = cls(
            employer=employer,
            pay_codes=pay_codes,
            csv_mappings=csv_mappings,
            departments=departments,
            rti=rti,
            users=users,
            hmrc_notice_settings=hmrc_notice_settings,
            pay_options=pay_options,
        )

        return employer_defaults



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="StatutoryPay")


@attr.s(auto_attribs=True)
class StatutoryPay:
    """Part of the TaxYearConfig that our engine uses to calculate Statutory Pay.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            weekly_parental_leave_amount (Union[Unset, float]): [readonly]
            weekly_sick_pay_amount (Union[Unset, float]): [readonly]
            awe_eligibility_threshold (Union[Unset, float]):
    """

    weekly_parental_leave_amount: Union[Unset, float] = UNSET
    weekly_sick_pay_amount: Union[Unset, float] = UNSET
    awe_eligibility_threshold: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        weekly_parental_leave_amount = self.weekly_parental_leave_amount
        weekly_sick_pay_amount = self.weekly_sick_pay_amount
        awe_eligibility_threshold = self.awe_eligibility_threshold

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if weekly_parental_leave_amount is not UNSET:
            field_dict["weeklyParentalLeaveAmount"] = weekly_parental_leave_amount
        if weekly_sick_pay_amount is not UNSET:
            field_dict["weeklySickPayAmount"] = weekly_sick_pay_amount
        if awe_eligibility_threshold is not UNSET:
            field_dict["aweEligibilityThreshold"] = awe_eligibility_threshold

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        weekly_parental_leave_amount = d.pop("weeklyParentalLeaveAmount", UNSET)

        weekly_sick_pay_amount = d.pop("weeklySickPayAmount", UNSET)

        awe_eligibility_threshold = d.pop("aweEligibilityThreshold", UNSET)

        statutory_pay = cls(
            weekly_parental_leave_amount=weekly_parental_leave_amount,
            weekly_sick_pay_amount=weekly_sick_pay_amount,
            awe_eligibility_threshold=awe_eligibility_threshold,
        )

        return statutory_pay



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="DpsSettings")


@attr.s(auto_attribs=True)
class DpsSettings:
    """This model is used to set an employers settings for HMRCs Data Provisioning Service

    Attributes:
        enabled (Union[Unset, bool]): If enabled, we'll automatically retrieve DPS notices from HMRC for you
        retrieve_from (Union[Unset, datetime.date]): The date from which notices should be retrieved
        auto_apply (Union[Unset, bool]): If enabled, we'll automatically apply DPSNotices before starting a payrun that
            covers the EffectiveDate
        last_checked (Union[Unset, None, datetime.date]): [readonly] The time we last checked for notices
        error (Union[Unset, None, str]): [readonly] If we received an error from HMRC when checking for notices, it'll
            be displayed here
    """

    enabled: Union[Unset, bool] = UNSET
    retrieve_from: Union[Unset, datetime.date] = UNSET
    auto_apply: Union[Unset, bool] = UNSET
    last_checked: Union[Unset, None, datetime.date] = UNSET
    error: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled
        retrieve_from: Union[Unset, str] = UNSET
        if not isinstance(self.retrieve_from, Unset):
            retrieve_from = self.retrieve_from.isoformat()

        auto_apply = self.auto_apply
        last_checked: Union[Unset, None, str] = UNSET
        if not isinstance(self.last_checked, Unset):
            last_checked = self.last_checked.isoformat() if self.last_checked else None

        error = self.error

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if retrieve_from is not UNSET:
            field_dict["retrieveFrom"] = retrieve_from
        if auto_apply is not UNSET:
            field_dict["autoApply"] = auto_apply
        if last_checked is not UNSET:
            field_dict["lastChecked"] = last_checked
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enabled = d.pop("enabled", UNSET)

        _retrieve_from = d.pop("retrieveFrom", UNSET)
        retrieve_from: Union[Unset, datetime.date]
        if isinstance(_retrieve_from, Unset):
            retrieve_from = UNSET
        else:
            retrieve_from = isoparse(_retrieve_from).date()

        auto_apply = d.pop("autoApply", UNSET)

        _last_checked = d.pop("lastChecked", UNSET)
        last_checked: Union[Unset, None, datetime.date]
        if _last_checked is None:
            last_checked = None
        elif isinstance(_last_checked, Unset):
            last_checked = UNSET
        else:
            last_checked = isoparse(_last_checked).date()

        error = d.pop("error", UNSET)

        dps_settings = cls(
            enabled=enabled,
            retrieve_from=retrieve_from,
            auto_apply=auto_apply,
            last_checked=last_checked,
            error=error,
        )

        return dps_settings



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.ae_status import AeStatus
from ..models.pension_contribution_level_type import PensionContributionLevelType
from ..models.pension_scheme import PensionScheme
from ..models.teachers_pension_details import TeachersPensionDetails
from ..models.worker_group import WorkerGroup
from ..types import UNSET, Unset

T = TypeVar("T", bound="Pension")


@attr.s(auto_attribs=True)
class Pension:
    """
    Attributes:
        contribution_level_type (Union[Unset, PensionContributionLevelType]):
        id (Union[Unset, str]): [readonly] The unique id of the object
        pension_scheme_id (Union[Unset, str]):
        pension_scheme (Union[Unset, PensionScheme]):
        worker_group_id (Union[Unset, str]):
        worker_group (Union[Unset, WorkerGroup]):
        start_date (Union[Unset, datetime.date]):
        member_reference_number (Union[Unset, None, str]):
        override_contributions (Union[Unset, bool]): If this is set to true then the Contributions levels set for the
            WorkerGroup will be replaced with the values provided here
        employee_contribution (Union[Unset, float]): The amount the employee contributes towards the pension. Could be a
            percentage or a fixed amount depending on EmployeeContributionIsPercentage.
            This is read-only if OverrideContributions is false
        employee_contribution_is_percentage (Union[Unset, bool]): Determines whether the Value of the
            EmployeeContribution is a fixed amount or a percentage,
        employer_contribution (Union[Unset, float]): The amount the employer contributes towards the pension. Could be a
            percentage or a fixed amount depending on EmployerContributionIsPercentage.
            This is read-only if OverrideContributions is false
        employer_contribution_is_percentage (Union[Unset, bool]): Determines whether the Value of the
            EmployerContribution is a fixed amount or a percentage,
        employer_contribution_top_up_percentage (Union[Unset, float]): Increase Employer Contribution by this percentage
            of the Employee Contribution
        is_ae_qualifying_scheme (Union[Unset, bool]): [readonly] Whether or not the associated PensionScheme is a
            Qualifying Scheme for AutoEnrolment
        is_teachers_pension (Union[Unset, bool]): [readonly] Whether or not the associated PensionScheme is a Teachers'
            Pension (determined by its CsvFormat)
        ae_status_at_joining (Union[Unset, AeStatus]):
        external_employee_id (Union[Unset, None, str]): [readonly]
        additional_voluntary_contribution (Union[Unset, float]): Any additional voluntary amount the employer
            contributes towards the pension. Could be a percentage or a fixed amount depending on AvcIsPercentage.
        avc_is_percentage (Union[Unset, bool]): Determines whether the Value of the Additional Voluntary Contribution is
            a fixed amount or a percentage,
        exit_via_provider (Union[Unset, bool]):
        teachers_pension_details (Union[Unset, TeachersPensionDetails]): Used to represent additional information needed
            for
            Teachers' Pensions
        forced_tier (Union[Unset, None, str]): If the WorkerGroup ContributionLevelType is a Tiered Scheme then you can
            specify the name of the tier to force the employee on to.
            If none is specified then the Tier is determined by the earnings in the period
        force_enrolment (Union[Unset, bool]): If the PensionScheme is connected to an ExternalDataProvider that supports
            enrolment then setting this to true will force this employee to be enrolled with the next submission.
        auto_enrolled (Union[Unset, bool]): [readonly] Is True if the employee joined this Pension due to an
            AutoEnrolment action
    """

    contribution_level_type: Union[Unset, PensionContributionLevelType] = UNSET
    id: Union[Unset, str] = UNSET
    pension_scheme_id: Union[Unset, str] = UNSET
    pension_scheme: Union[Unset, PensionScheme] = UNSET
    worker_group_id: Union[Unset, str] = UNSET
    worker_group: Union[Unset, WorkerGroup] = UNSET
    start_date: Union[Unset, datetime.date] = UNSET
    member_reference_number: Union[Unset, None, str] = UNSET
    override_contributions: Union[Unset, bool] = UNSET
    employee_contribution: Union[Unset, float] = UNSET
    employee_contribution_is_percentage: Union[Unset, bool] = UNSET
    employer_contribution: Union[Unset, float] = UNSET
    employer_contribution_is_percentage: Union[Unset, bool] = UNSET
    employer_contribution_top_up_percentage: Union[Unset, float] = UNSET
    is_ae_qualifying_scheme: Union[Unset, bool] = UNSET
    is_teachers_pension: Union[Unset, bool] = UNSET
    ae_status_at_joining: Union[Unset, AeStatus] = UNSET
    external_employee_id: Union[Unset, None, str] = UNSET
    additional_voluntary_contribution: Union[Unset, float] = UNSET
    avc_is_percentage: Union[Unset, bool] = UNSET
    exit_via_provider: Union[Unset, bool] = UNSET
    teachers_pension_details: Union[Unset, TeachersPensionDetails] = UNSET
    forced_tier: Union[Unset, None, str] = UNSET
    force_enrolment: Union[Unset, bool] = UNSET
    auto_enrolled: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        contribution_level_type: Union[Unset, str] = UNSET
        if not isinstance(self.contribution_level_type, Unset):
            contribution_level_type = self.contribution_level_type.value

        id = self.id
        pension_scheme_id = self.pension_scheme_id
        pension_scheme: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pension_scheme, Unset):
            pension_scheme = self.pension_scheme.to_dict()

        worker_group_id = self.worker_group_id
        worker_group: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.worker_group, Unset):
            worker_group = self.worker_group.to_dict()

        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        member_reference_number = self.member_reference_number
        override_contributions = self.override_contributions
        employee_contribution = self.employee_contribution
        employee_contribution_is_percentage = self.employee_contribution_is_percentage
        employer_contribution = self.employer_contribution
        employer_contribution_is_percentage = self.employer_contribution_is_percentage
        employer_contribution_top_up_percentage = (
            self.employer_contribution_top_up_percentage
        )
        is_ae_qualifying_scheme = self.is_ae_qualifying_scheme
        is_teachers_pension = self.is_teachers_pension
        ae_status_at_joining: Union[Unset, str] = UNSET
        if not isinstance(self.ae_status_at_joining, Unset):
            ae_status_at_joining = self.ae_status_at_joining.value

        external_employee_id = self.external_employee_id
        additional_voluntary_contribution = self.additional_voluntary_contribution
        avc_is_percentage = self.avc_is_percentage
        exit_via_provider = self.exit_via_provider
        teachers_pension_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.teachers_pension_details, Unset):
            teachers_pension_details = self.teachers_pension_details.to_dict()

        forced_tier = self.forced_tier
        force_enrolment = self.force_enrolment
        auto_enrolled = self.auto_enrolled

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if contribution_level_type is not UNSET:
            field_dict["contributionLevelType"] = contribution_level_type
        if id is not UNSET:
            field_dict["id"] = id
        if pension_scheme_id is not UNSET:
            field_dict["pensionSchemeId"] = pension_scheme_id
        if pension_scheme is not UNSET:
            field_dict["pensionScheme"] = pension_scheme
        if worker_group_id is not UNSET:
            field_dict["workerGroupId"] = worker_group_id
        if worker_group is not UNSET:
            field_dict["workerGroup"] = worker_group
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if member_reference_number is not UNSET:
            field_dict["memberReferenceNumber"] = member_reference_number
        if override_contributions is not UNSET:
            field_dict["overrideContributions"] = override_contributions
        if employee_contribution is not UNSET:
            field_dict["employeeContribution"] = employee_contribution
        if employee_contribution_is_percentage is not UNSET:
            field_dict[
                "employeeContributionIsPercentage"
            ] = employee_contribution_is_percentage
        if employer_contribution is not UNSET:
            field_dict["employerContribution"] = employer_contribution
        if employer_contribution_is_percentage is not UNSET:
            field_dict[
                "employerContributionIsPercentage"
            ] = employer_contribution_is_percentage
        if employer_contribution_top_up_percentage is not UNSET:
            field_dict[
                "employerContributionTopUpPercentage"
            ] = employer_contribution_top_up_percentage
        if is_ae_qualifying_scheme is not UNSET:
            field_dict["isAeQualifyingScheme"] = is_ae_qualifying_scheme
        if is_teachers_pension is not UNSET:
            field_dict["isTeachersPension"] = is_teachers_pension
        if ae_status_at_joining is not UNSET:
            field_dict["aeStatusAtJoining"] = ae_status_at_joining
        if external_employee_id is not UNSET:
            field_dict["externalEmployeeId"] = external_employee_id
        if additional_voluntary_contribution is not UNSET:
            field_dict[
                "additionalVoluntaryContribution"
            ] = additional_voluntary_contribution
        if avc_is_percentage is not UNSET:
            field_dict["avcIsPercentage"] = avc_is_percentage
        if exit_via_provider is not UNSET:
            field_dict["exitViaProvider"] = exit_via_provider
        if teachers_pension_details is not UNSET:
            field_dict["teachersPensionDetails"] = teachers_pension_details
        if forced_tier is not UNSET:
            field_dict["forcedTier"] = forced_tier
        if force_enrolment is not UNSET:
            field_dict["forceEnrolment"] = force_enrolment
        if auto_enrolled is not UNSET:
            field_dict["autoEnrolled"] = auto_enrolled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _contribution_level_type = d.pop("contributionLevelType", UNSET)
        contribution_level_type: Union[Unset, PensionContributionLevelType]
        if isinstance(_contribution_level_type, Unset):
            contribution_level_type = UNSET
        else:
            contribution_level_type = PensionContributionLevelType(
                _contribution_level_type
            )

        id = d.pop("id", UNSET)

        pension_scheme_id = d.pop("pensionSchemeId", UNSET)

        _pension_scheme = d.pop("pensionScheme", UNSET)
        pension_scheme: Union[Unset, PensionScheme]
        if isinstance(_pension_scheme, Unset):
            pension_scheme = UNSET
        else:
            pension_scheme = PensionScheme.from_dict(_pension_scheme)

        worker_group_id = d.pop("workerGroupId", UNSET)

        _worker_group = d.pop("workerGroup", UNSET)
        worker_group: Union[Unset, WorkerGroup]
        if isinstance(_worker_group, Unset):
            worker_group = UNSET
        else:
            worker_group = WorkerGroup.from_dict(_worker_group)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.date]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        member_reference_number = d.pop("memberReferenceNumber", UNSET)

        override_contributions = d.pop("overrideContributions", UNSET)

        employee_contribution = d.pop("employeeContribution", UNSET)

        employee_contribution_is_percentage = d.pop(
            "employeeContributionIsPercentage", UNSET
        )

        employer_contribution = d.pop("employerContribution", UNSET)

        employer_contribution_is_percentage = d.pop(
            "employerContributionIsPercentage", UNSET
        )

        employer_contribution_top_up_percentage = d.pop(
            "employerContributionTopUpPercentage", UNSET
        )

        is_ae_qualifying_scheme = d.pop("isAeQualifyingScheme", UNSET)

        is_teachers_pension = d.pop("isTeachersPension", UNSET)

        _ae_status_at_joining = d.pop("aeStatusAtJoining", UNSET)
        ae_status_at_joining: Union[Unset, AeStatus]
        if isinstance(_ae_status_at_joining, Unset):
            ae_status_at_joining = UNSET
        else:
            ae_status_at_joining = AeStatus(_ae_status_at_joining)

        external_employee_id = d.pop("externalEmployeeId", UNSET)

        additional_voluntary_contribution = d.pop(
            "additionalVoluntaryContribution", UNSET
        )

        avc_is_percentage = d.pop("avcIsPercentage", UNSET)

        exit_via_provider = d.pop("exitViaProvider", UNSET)

        _teachers_pension_details = d.pop("teachersPensionDetails", UNSET)
        teachers_pension_details: Union[Unset, TeachersPensionDetails]
        if isinstance(_teachers_pension_details, Unset):
            teachers_pension_details = UNSET
        else:
            teachers_pension_details = TeachersPensionDetails.from_dict(
                _teachers_pension_details
            )

        forced_tier = d.pop("forcedTier", UNSET)

        force_enrolment = d.pop("forceEnrolment", UNSET)

        auto_enrolled = d.pop("autoEnrolled", UNSET)

        pension = cls(
            contribution_level_type=contribution_level_type,
            id=id,
            pension_scheme_id=pension_scheme_id,
            pension_scheme=pension_scheme,
            worker_group_id=worker_group_id,
            worker_group=worker_group,
            start_date=start_date,
            member_reference_number=member_reference_number,
            override_contributions=override_contributions,
            employee_contribution=employee_contribution,
            employee_contribution_is_percentage=employee_contribution_is_percentage,
            employer_contribution=employer_contribution,
            employer_contribution_is_percentage=employer_contribution_is_percentage,
            employer_contribution_top_up_percentage=employer_contribution_top_up_percentage,
            is_ae_qualifying_scheme=is_ae_qualifying_scheme,
            is_teachers_pension=is_teachers_pension,
            ae_status_at_joining=ae_status_at_joining,
            external_employee_id=external_employee_id,
            additional_voluntary_contribution=additional_voluntary_contribution,
            avc_is_percentage=avc_is_percentage,
            exit_via_provider=exit_via_provider,
            teachers_pension_details=teachers_pension_details,
            forced_tier=forced_tier,
            force_enrolment=force_enrolment,
            auto_enrolled=auto_enrolled,
        )

        return pension



from typing import Any, Dict, Optional

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.eps import Eps
from ...models.tax_year import TaxYear
from ...types import Response


def _get_kwargs(
    employer_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: Eps,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/rti/eps/{taxYear}".format(
        client.base_url, employerId=employer_id, taxYear=tax_year
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Eps]:
    if response.status_code == 201:
        response_201 = Eps.from_dict(response.json())

        return response_201
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[Eps]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: Eps,
) -> Response[Eps]:
    """Create EPS

     Creates a new Employer Payment Summary

    Args:
        employer_id (str):
        tax_year (TaxYear):
        json_body (Eps):

    Returns:
        Response[Eps]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: Eps,
) -> Optional[Eps]:
    """Create EPS

     Creates a new Employer Payment Summary

    Args:
        employer_id (str):
        tax_year (TaxYear):
        json_body (Eps):

    Returns:
        Response[Eps]
    """

    return sync_detailed(
        employer_id=employer_id,
        tax_year=tax_year,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: Eps,
) -> Response[Eps]:
    """Create EPS

     Creates a new Employer Payment Summary

    Args:
        employer_id (str):
        tax_year (TaxYear):
        json_body (Eps):

    Returns:
        Response[Eps]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: Eps,
) -> Optional[Eps]:
    """Create EPS

     Creates a new Employer Payment Summary

    Args:
        employer_id (str):
        tax_year (TaxYear):
        json_body (Eps):

    Returns:
        Response[Eps]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            tax_year=tax_year,
            client=client,
            json_body=json_body,
        )
    ).parsed



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.bank_details import BankDetails
from ..models.item import Item
from ..models.pay_method import PayMethod
from ..models.payee_type import PayeeType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayRunPayment")


@attr.s(auto_attribs=True)
class PayRunPayment:
    """
    Attributes:
        type (Union[Unset, PayeeType]):
        payee (Union[Unset, Item]):
        date (Union[Unset, datetime.date]): [readonly] The date the payment is to be made
        method (Union[Unset, PayMethod]):
        amount (Union[Unset, float]): [readonly] The amount to pay
        reference (Union[Unset, None, str]): [readonly] The period the payment is for
        bank_details (Union[Unset, BankDetails]):
        bacs_hash (Union[Unset, None, str]): [readonly] if paying by Credit and you've enabled BacsHash then this will
            contain the BacsHash included on the FPS
        bacs_sub_reference (Union[Unset, None, str]): [readonly] if paying by Credit and you've enabled BacsHash then
            this will contain the BacsSubReference included on the FPS
    """

    type: Union[Unset, PayeeType] = UNSET
    payee: Union[Unset, Item] = UNSET
    date: Union[Unset, datetime.date] = UNSET
    method: Union[Unset, PayMethod] = UNSET
    amount: Union[Unset, float] = UNSET
    reference: Union[Unset, None, str] = UNSET
    bank_details: Union[Unset, BankDetails] = UNSET
    bacs_hash: Union[Unset, None, str] = UNSET
    bacs_sub_reference: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        payee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.payee, Unset):
            payee = self.payee.to_dict()

        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        method: Union[Unset, str] = UNSET
        if not isinstance(self.method, Unset):
            method = self.method.value

        amount = self.amount
        reference = self.reference
        bank_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bank_details, Unset):
            bank_details = self.bank_details.to_dict()

        bacs_hash = self.bacs_hash
        bacs_sub_reference = self.bacs_sub_reference

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if payee is not UNSET:
            field_dict["payee"] = payee
        if date is not UNSET:
            field_dict["date"] = date
        if method is not UNSET:
            field_dict["method"] = method
        if amount is not UNSET:
            field_dict["amount"] = amount
        if reference is not UNSET:
            field_dict["reference"] = reference
        if bank_details is not UNSET:
            field_dict["bankDetails"] = bank_details
        if bacs_hash is not UNSET:
            field_dict["bacsHash"] = bacs_hash
        if bacs_sub_reference is not UNSET:
            field_dict["bacsSubReference"] = bacs_sub_reference

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, PayeeType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = PayeeType(_type)

        _payee = d.pop("payee", UNSET)
        payee: Union[Unset, Item]
        if isinstance(_payee, Unset):
            payee = UNSET
        else:
            payee = Item.from_dict(_payee)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        _method = d.pop("method", UNSET)
        method: Union[Unset, PayMethod]
        if isinstance(_method, Unset):
            method = UNSET
        else:
            method = PayMethod(_method)

        amount = d.pop("amount", UNSET)

        reference = d.pop("reference", UNSET)

        _bank_details = d.pop("bankDetails", UNSET)
        bank_details: Union[Unset, BankDetails]
        if isinstance(_bank_details, Unset):
            bank_details = UNSET
        else:
            bank_details = BankDetails.from_dict(_bank_details)

        bacs_hash = d.pop("bacsHash", UNSET)

        bacs_sub_reference = d.pop("bacsSubReference", UNSET)

        pay_run_payment = cls(
            type=type,
            payee=payee,
            date=date,
            method=method,
            amount=amount,
            reference=reference,
            bank_details=bank_details,
            bacs_hash=bacs_hash,
            bacs_sub_reference=bacs_sub_reference,
        )

        return pay_run_payment



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.background_task_status import BackgroundTaskStatus
from ..models.pay_periods import PayPeriods
from ..models.pay_run_summary_line import PayRunSummaryLine
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayRunJournal")


@attr.s(auto_attribs=True)
class PayRunJournal:
    """
    Attributes:
        lines (Union[Unset, None, List[PayRunSummaryLine]]):
        date (Union[Unset, datetime.date]): [readonly] The PaymentDate from the PayRun
        title (Union[Unset, None, str]): [readonly] The status of the task for posting this journal to an
            ExternalDataProvider
        status (Union[Unset, BackgroundTaskStatus]):
        status_message (Union[Unset, None, str]): [readonly] A message to elaborate on the Status
        link (Union[Unset, None, str]): [readonly] If available, a link to the journal in the ExternalDataProvider
        tax_year (Union[Unset, TaxYear]):
        pay_period (Union[Unset, PayPeriods]):
        ordinal (Union[Unset, int]): [readonly]
        period (Union[Unset, int]): [readonly]
    """

    lines: Union[Unset, None, List[PayRunSummaryLine]] = UNSET
    date: Union[Unset, datetime.date] = UNSET
    title: Union[Unset, None, str] = UNSET
    status: Union[Unset, BackgroundTaskStatus] = UNSET
    status_message: Union[Unset, None, str] = UNSET
    link: Union[Unset, None, str] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    ordinal: Union[Unset, int] = UNSET
    period: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.lines, Unset):
            if self.lines is None:
                lines = None
            else:
                lines = []
                for lines_item_data in self.lines:
                    lines_item = lines_item_data.to_dict()

                    lines.append(lines_item)

        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        title = self.title
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        status_message = self.status_message
        link = self.link
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        ordinal = self.ordinal
        period = self.period

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if lines is not UNSET:
            field_dict["lines"] = lines
        if date is not UNSET:
            field_dict["date"] = date
        if title is not UNSET:
            field_dict["title"] = title
        if status is not UNSET:
            field_dict["status"] = status
        if status_message is not UNSET:
            field_dict["statusMessage"] = status_message
        if link is not UNSET:
            field_dict["link"] = link
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if period is not UNSET:
            field_dict["period"] = period

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        lines = []
        _lines = d.pop("lines", UNSET)
        for lines_item_data in _lines or []:
            lines_item = PayRunSummaryLine.from_dict(lines_item_data)

            lines.append(lines_item)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        title = d.pop("title", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, BackgroundTaskStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = BackgroundTaskStatus(_status)

        status_message = d.pop("statusMessage", UNSET)

        link = d.pop("link", UNSET)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        ordinal = d.pop("ordinal", UNSET)

        period = d.pop("period", UNSET)

        pay_run_journal = cls(
            lines=lines,
            date=date,
            title=title,
            status=status,
            status_message=status_message,
            link=link,
            tax_year=tax_year,
            pay_period=pay_period,
            ordinal=ordinal,
            period=period,
        )

        return pay_run_journal



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.address import Address
from ..models.papdis_version import PapdisVersion
from ..models.pension_csv_format import PensionCsvFormat
from ..types import UNSET, Unset

T = TypeVar("T", bound="PensionProvider")


@attr.s(auto_attribs=True)
class PensionProvider:
    """
    Attributes:
        name (str):
        account_no (Union[Unset, None, str]):
        portal (Union[Unset, None, str]):
        website (Union[Unset, None, str]):
        address (Union[Unset, Address]):
        telephone (Union[Unset, None, str]):
        papdis_version (Union[Unset, PapdisVersion]):
        papdis_provider_id (Union[Unset, None, str]):
        papdis_employer_id (Union[Unset, None, str]):
        csv_format (Union[Unset, PensionCsvFormat]):
        exclude_nil_paid_from_contributions (Union[Unset, bool]): If we're sending contributions to an external provider
            then we'll include all employees that were on the payrun.
            If you want to exclude employees that don't have any contributions to report then set this to true.
        pay_period_date_adjustment (Union[Unset, int]): If you need to adjust the reported dates of the contributions
            then you can do so by setting a non-zero value here.
            A negative value of will move the date back in time.
        misc_boolean_1 (Union[Unset, bool]): This field has different uses dependent on the ExternalDataProvider, if
            any.
            For Nest, it indicates whether or not contributions are reported as "Tax Weekly/Monthly" rather than just
            "Weekly/Monthly"
        misc_boolean_2 (Union[Unset, bool]): This field has different uses dependent on the ExternalDataProvider, if
            any.
            For Nest, it indicates whether or not to approve payments after submitting contributions
        misc_string_1 (Union[Unset, None, str]): This field has different uses dependent on the ExternalDataProvider, if
            any.
            For Nest, it dictates the PaymentSource.
        misc_string_2 (Union[Unset, None, str]): This field has different uses dependent on the ExternalDataProvider, if
            any.
        opt_out_window (Union[Unset, int]): The number of days or months that an employee has to Opt out after being
            enrolled
        opt_out_window_is_months (Union[Unset, bool]): Determines whether the value given for OptOutWindow is in Months
            (true) or days (false)
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    name: str
    account_no: Union[Unset, None, str] = UNSET
    portal: Union[Unset, None, str] = UNSET
    website: Union[Unset, None, str] = UNSET
    address: Union[Unset, Address] = UNSET
    telephone: Union[Unset, None, str] = UNSET
    papdis_version: Union[Unset, PapdisVersion] = UNSET
    papdis_provider_id: Union[Unset, None, str] = UNSET
    papdis_employer_id: Union[Unset, None, str] = UNSET
    csv_format: Union[Unset, PensionCsvFormat] = UNSET
    exclude_nil_paid_from_contributions: Union[Unset, bool] = UNSET
    pay_period_date_adjustment: Union[Unset, int] = UNSET
    misc_boolean_1: Union[Unset, bool] = UNSET
    misc_boolean_2: Union[Unset, bool] = UNSET
    misc_string_1: Union[Unset, None, str] = UNSET
    misc_string_2: Union[Unset, None, str] = UNSET
    opt_out_window: Union[Unset, int] = UNSET
    opt_out_window_is_months: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        account_no = self.account_no
        portal = self.portal
        website = self.website
        address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        telephone = self.telephone
        papdis_version: Union[Unset, str] = UNSET
        if not isinstance(self.papdis_version, Unset):
            papdis_version = self.papdis_version.value

        papdis_provider_id = self.papdis_provider_id
        papdis_employer_id = self.papdis_employer_id
        csv_format: Union[Unset, str] = UNSET
        if not isinstance(self.csv_format, Unset):
            csv_format = self.csv_format.value

        exclude_nil_paid_from_contributions = self.exclude_nil_paid_from_contributions
        pay_period_date_adjustment = self.pay_period_date_adjustment
        misc_boolean_1 = self.misc_boolean_1
        misc_boolean_2 = self.misc_boolean_2
        misc_string_1 = self.misc_string_1
        misc_string_2 = self.misc_string_2
        opt_out_window = self.opt_out_window
        opt_out_window_is_months = self.opt_out_window_is_months
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if account_no is not UNSET:
            field_dict["accountNo"] = account_no
        if portal is not UNSET:
            field_dict["portal"] = portal
        if website is not UNSET:
            field_dict["website"] = website
        if address is not UNSET:
            field_dict["address"] = address
        if telephone is not UNSET:
            field_dict["telephone"] = telephone
        if papdis_version is not UNSET:
            field_dict["papdisVersion"] = papdis_version
        if papdis_provider_id is not UNSET:
            field_dict["papdisProviderId"] = papdis_provider_id
        if papdis_employer_id is not UNSET:
            field_dict["papdisEmployerId"] = papdis_employer_id
        if csv_format is not UNSET:
            field_dict["csvFormat"] = csv_format
        if exclude_nil_paid_from_contributions is not UNSET:
            field_dict[
                "excludeNilPaidFromContributions"
            ] = exclude_nil_paid_from_contributions
        if pay_period_date_adjustment is not UNSET:
            field_dict["payPeriodDateAdjustment"] = pay_period_date_adjustment
        if misc_boolean_1 is not UNSET:
            field_dict["miscBoolean1"] = misc_boolean_1
        if misc_boolean_2 is not UNSET:
            field_dict["miscBoolean2"] = misc_boolean_2
        if misc_string_1 is not UNSET:
            field_dict["miscString1"] = misc_string_1
        if misc_string_2 is not UNSET:
            field_dict["miscString2"] = misc_string_2
        if opt_out_window is not UNSET:
            field_dict["optOutWindow"] = opt_out_window
        if opt_out_window_is_months is not UNSET:
            field_dict["optOutWindowIsMonths"] = opt_out_window_is_months
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        account_no = d.pop("accountNo", UNSET)

        portal = d.pop("portal", UNSET)

        website = d.pop("website", UNSET)

        _address = d.pop("address", UNSET)
        address: Union[Unset, Address]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = Address.from_dict(_address)

        telephone = d.pop("telephone", UNSET)

        _papdis_version = d.pop("papdisVersion", UNSET)
        papdis_version: Union[Unset, PapdisVersion]
        if isinstance(_papdis_version, Unset):
            papdis_version = UNSET
        else:
            papdis_version = PapdisVersion(_papdis_version)

        papdis_provider_id = d.pop("papdisProviderId", UNSET)

        papdis_employer_id = d.pop("papdisEmployerId", UNSET)

        _csv_format = d.pop("csvFormat", UNSET)
        csv_format: Union[Unset, PensionCsvFormat]
        if isinstance(_csv_format, Unset):
            csv_format = UNSET
        else:
            csv_format = PensionCsvFormat(_csv_format)

        exclude_nil_paid_from_contributions = d.pop(
            "excludeNilPaidFromContributions", UNSET
        )

        pay_period_date_adjustment = d.pop("payPeriodDateAdjustment", UNSET)

        misc_boolean_1 = d.pop("miscBoolean1", UNSET)

        misc_boolean_2 = d.pop("miscBoolean2", UNSET)

        misc_string_1 = d.pop("miscString1", UNSET)

        misc_string_2 = d.pop("miscString2", UNSET)

        opt_out_window = d.pop("optOutWindow", UNSET)

        opt_out_window_is_months = d.pop("optOutWindowIsMonths", UNSET)

        id = d.pop("id", UNSET)

        pension_provider = cls(
            name=name,
            account_no=account_no,
            portal=portal,
            website=website,
            address=address,
            telephone=telephone,
            papdis_version=papdis_version,
            papdis_provider_id=papdis_provider_id,
            papdis_employer_id=papdis_employer_id,
            csv_format=csv_format,
            exclude_nil_paid_from_contributions=exclude_nil_paid_from_contributions,
            pay_period_date_adjustment=pay_period_date_adjustment,
            misc_boolean_1=misc_boolean_1,
            misc_boolean_2=misc_boolean_2,
            misc_string_1=misc_string_1,
            misc_string_2=misc_string_2,
            opt_out_window=opt_out_window,
            opt_out_window_is_months=opt_out_window_is_months,
            id=id,
        )

        return pension_provider



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.gross_to_net_report import GrossToNetReport
from ..models.gross_to_net_report_line import GrossToNetReportLine
from ..models.report import Report
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="VarianceReport")


@attr.s(auto_attribs=True)
class VarianceReport:
    """
    Attributes:
        show_difference_as_percentage (Union[Unset, bool]):
        minimum_change_percentage (Union[Unset, float]):
        primary (Union[Unset, GrossToNetReport]):
        secondary (Union[Unset, GrossToNetReport]):
        joiners (Union[Unset, None, List[GrossToNetReportLine]]):
        leavers (Union[Unset, None, List[GrossToNetReportLine]]):
        has_departments (Union[Unset, bool]):
        common_lines (Union[Unset, None, List[GrossToNetReportLine]]):
        has_variances (Union[Unset, bool]):
        report (Union[Unset, Report]):
        tax_year (Union[Unset, TaxYear]):
    """

    show_difference_as_percentage: Union[Unset, bool] = UNSET
    minimum_change_percentage: Union[Unset, float] = UNSET
    primary: Union[Unset, GrossToNetReport] = UNSET
    secondary: Union[Unset, GrossToNetReport] = UNSET
    joiners: Union[Unset, None, List[GrossToNetReportLine]] = UNSET
    leavers: Union[Unset, None, List[GrossToNetReportLine]] = UNSET
    has_departments: Union[Unset, bool] = UNSET
    common_lines: Union[Unset, None, List[GrossToNetReportLine]] = UNSET
    has_variances: Union[Unset, bool] = UNSET
    report: Union[Unset, Report] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        show_difference_as_percentage = self.show_difference_as_percentage
        minimum_change_percentage = self.minimum_change_percentage
        primary: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.primary, Unset):
            primary = self.primary.to_dict()

        secondary: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.secondary, Unset):
            secondary = self.secondary.to_dict()

        joiners: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.joiners, Unset):
            if self.joiners is None:
                joiners = None
            else:
                joiners = []
                for joiners_item_data in self.joiners:
                    joiners_item = joiners_item_data.to_dict()

                    joiners.append(joiners_item)

        leavers: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.leavers, Unset):
            if self.leavers is None:
                leavers = None
            else:
                leavers = []
                for leavers_item_data in self.leavers:
                    leavers_item = leavers_item_data.to_dict()

                    leavers.append(leavers_item)

        has_departments = self.has_departments
        common_lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.common_lines, Unset):
            if self.common_lines is None:
                common_lines = None
            else:
                common_lines = []
                for common_lines_item_data in self.common_lines:
                    common_lines_item = common_lines_item_data.to_dict()

                    common_lines.append(common_lines_item)

        has_variances = self.has_variances
        report: Union[Unset, str] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.value

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if show_difference_as_percentage is not UNSET:
            field_dict["showDifferenceAsPercentage"] = show_difference_as_percentage
        if minimum_change_percentage is not UNSET:
            field_dict["minimumChangePercentage"] = minimum_change_percentage
        if primary is not UNSET:
            field_dict["primary"] = primary
        if secondary is not UNSET:
            field_dict["secondary"] = secondary
        if joiners is not UNSET:
            field_dict["joiners"] = joiners
        if leavers is not UNSET:
            field_dict["leavers"] = leavers
        if has_departments is not UNSET:
            field_dict["hasDepartments"] = has_departments
        if common_lines is not UNSET:
            field_dict["commonLines"] = common_lines
        if has_variances is not UNSET:
            field_dict["hasVariances"] = has_variances
        if report is not UNSET:
            field_dict["report"] = report
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        show_difference_as_percentage = d.pop("showDifferenceAsPercentage", UNSET)

        minimum_change_percentage = d.pop("minimumChangePercentage", UNSET)

        _primary = d.pop("primary", UNSET)
        primary: Union[Unset, GrossToNetReport]
        if isinstance(_primary, Unset):
            primary = UNSET
        else:
            primary = GrossToNetReport.from_dict(_primary)

        _secondary = d.pop("secondary", UNSET)
        secondary: Union[Unset, GrossToNetReport]
        if isinstance(_secondary, Unset):
            secondary = UNSET
        else:
            secondary = GrossToNetReport.from_dict(_secondary)

        joiners = []
        _joiners = d.pop("joiners", UNSET)
        for joiners_item_data in _joiners or []:
            joiners_item = GrossToNetReportLine.from_dict(joiners_item_data)

            joiners.append(joiners_item)

        leavers = []
        _leavers = d.pop("leavers", UNSET)
        for leavers_item_data in _leavers or []:
            leavers_item = GrossToNetReportLine.from_dict(leavers_item_data)

            leavers.append(leavers_item)

        has_departments = d.pop("hasDepartments", UNSET)

        common_lines = []
        _common_lines = d.pop("commonLines", UNSET)
        for common_lines_item_data in _common_lines or []:
            common_lines_item = GrossToNetReportLine.from_dict(common_lines_item_data)

            common_lines.append(common_lines_item)

        has_variances = d.pop("hasVariances", UNSET)

        _report = d.pop("report", UNSET)
        report: Union[Unset, Report]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = Report(_report)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        variance_report = cls(
            show_difference_as_percentage=show_difference_as_percentage,
            minimum_change_percentage=minimum_change_percentage,
            primary=primary,
            secondary=secondary,
            joiners=joiners,
            leavers=leavers,
            has_departments=has_departments,
            common_lines=common_lines,
            has_variances=has_variances,
            report=report,
            tax_year=tax_year,
        )

        return variance_report



from enum import Enum


class BankPaymentInstructionsCsvFormat(str, Enum):
    STANDARDCSV = "StandardCsv"
    TELLEROO = "Telleroo"
    BARCLAYSBACS = "BarclaysBacs"
    SANTANDERBACS = "SantanderBacs"
    SIF = "Sif"
    REVOLUT = "Revolut"
    STANDARD18FASTERPAYMENTS = "Standard18FasterPayments"
    STANDARD18BACS = "Standard18Bacs"
    BANKLINE = "Bankline"
    BANKLINEBULK = "BanklineBulk"
    STANDARDCSVBACS = "StandardCsvBacs"
    LLOYDSMULTIPLESTANDARDCSVBACS = "LloydsMultipleStandardCsvBacs"
    LLOYDSV11CSVBACS = "LloydsV11CsvBacs"
    COOPBULKCSVBACS = "CoOpBulkCsvBacs"
    COOPFASTERPAYMENTSCSV = "CoOpFasterPaymentsCsv"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, Optional, Union, cast

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    u: Union[Unset, None, str] = UNSET,
    k: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/account/verify/respond".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["u"] = u

    params["k"] = k

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[bool]:
    if response.status_code == 200:
        response_200 = cast(bool, response.json())
        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[bool]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    u: Union[Unset, None, str] = UNSET,
    k: Union[Unset, None, str] = UNSET,
) -> Response[bool]:
    """Verify Email Address

     Used to process the link sent in an email to verify an email address.

    Args:
        u (Union[Unset, None, str]):
        k (Union[Unset, None, str]):

    Returns:
        Response[bool]
    """

    kwargs = _get_kwargs(
        client=client,
        u=u,
        k=k,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    u: Union[Unset, None, str] = UNSET,
    k: Union[Unset, None, str] = UNSET,
) -> Optional[bool]:
    """Verify Email Address

     Used to process the link sent in an email to verify an email address.

    Args:
        u (Union[Unset, None, str]):
        k (Union[Unset, None, str]):

    Returns:
        Response[bool]
    """

    return sync_detailed(
        client=client,
        u=u,
        k=k,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    u: Union[Unset, None, str] = UNSET,
    k: Union[Unset, None, str] = UNSET,
) -> Response[bool]:
    """Verify Email Address

     Used to process the link sent in an email to verify an email address.

    Args:
        u (Union[Unset, None, str]):
        k (Union[Unset, None, str]):

    Returns:
        Response[bool]
    """

    kwargs = _get_kwargs(
        client=client,
        u=u,
        k=k,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    u: Union[Unset, None, str] = UNSET,
    k: Union[Unset, None, str] = UNSET,
) -> Optional[bool]:
    """Verify Email Address

     Used to process the link sent in an email to verify an email address.

    Args:
        u (Union[Unset, None, str]):
        k (Union[Unset, None, str]):

    Returns:
        Response[bool]
    """

    return (
        await asyncio_detailed(
            client=client,
            u=u,
            k=k,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CarChargeRate")


@attr.s(auto_attribs=True)
class CarChargeRate:
    """Part of the TaxYearConfig that our engine uses to calculate charges for a Company Car.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            range_start (Union[Unset, int]):
            range_stop (Union[Unset, int]):
            rate (Union[Unset, float]):
    """

    range_start: Union[Unset, int] = UNSET
    range_stop: Union[Unset, int] = UNSET
    rate: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        range_start = self.range_start
        range_stop = self.range_stop
        rate = self.rate

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if range_start is not UNSET:
            field_dict["rangeStart"] = range_start
        if range_stop is not UNSET:
            field_dict["rangeStop"] = range_stop
        if rate is not UNSET:
            field_dict["rate"] = rate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        range_start = d.pop("rangeStart", UNSET)

        range_stop = d.pop("rangeStop", UNSET)

        rate = d.pop("rate", UNSET)

        car_charge_rate = cls(
            range_start=range_start,
            range_stop=range_stop,
            rate=rate,
        )

        return car_charge_rate



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NominalCodeMapping")


@attr.s(auto_attribs=True)
class NominalCodeMapping:
    """
    Attributes:
        nominal_code (Union[Unset, None, str]):
        nominal_name (Union[Unset, None, str]):
        external_id (Union[Unset, None, str]): If you will be using the API to post journals to an ExternalDataProvider,
            then this is the Id of the nominal code in the external system.
            If there's no mapping then this will have a value of "0"
        pay_code (Union[Unset, None, str]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    nominal_code: Union[Unset, None, str] = UNSET
    nominal_name: Union[Unset, None, str] = UNSET
    external_id: Union[Unset, None, str] = UNSET
    pay_code: Union[Unset, None, str] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        nominal_code = self.nominal_code
        nominal_name = self.nominal_name
        external_id = self.external_id
        pay_code = self.pay_code
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if nominal_code is not UNSET:
            field_dict["nominalCode"] = nominal_code
        if nominal_name is not UNSET:
            field_dict["nominalName"] = nominal_name
        if external_id is not UNSET:
            field_dict["externalId"] = external_id
        if pay_code is not UNSET:
            field_dict["payCode"] = pay_code
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        nominal_code = d.pop("nominalCode", UNSET)

        nominal_name = d.pop("nominalName", UNSET)

        external_id = d.pop("externalId", UNSET)

        pay_code = d.pop("payCode", UNSET)

        id = d.pop("id", UNSET)

        nominal_code_mapping = cls(
            nominal_code=nominal_code,
            nominal_name=nominal_name,
            external_id=external_id,
            pay_code=pay_code,
            id=id,
        )

        return nominal_code_mapping
